BEGIN;

-- enable the hstore extension
CREATE EXTENSION hstore;

-- new columns in member table for storing external authority data
ALTER TABLE "member" ADD COLUMN "authority" TEXT;
ALTER TABLE "member" ADD COLUMN "authority_data" hstore;

-- enforce a unique uid per authority and index for uid lookups
CREATE UNIQUE INDEX "member_authority_uid_idx"
  ON "member" ("authority", ("authority_data"->'uid'));

-- index for login lookups
CREATE INDEX "member_authority_data_login_idx"
  ON "member" (("authority_data"->'login'));

-- require a uid when account is authorized externally
ALTER TABLE "member" ADD CONSTRAINT "authority_requires_uid" 
  CHECK ("authority" ISNULL OR ("authority_data"->'uid') NOTNULL);

-- do not allow extra authority data unless an authority is used
ALTER TABLE "member" ADD CONSTRAINT "authority_data_requires_authority"
  CHECK ("authority" NOTNULL OR "authority_data" ISNULL);

-- new columns in session table for storing external authority data during
-- registration
ALTER TABLE "session" ADD COLUMN "authority" TEXT;
ALTER TABLE "session" ADD COLUMN "authority_data" hstore;

-- index for uid lookups
CREATE INDEX "member_session_authority_data_uid_idx"
  ON "session" (("authority_data"->'uid'));

-- index for login lookups
CREATE INDEX "member_session_authority_data_login_idx"
  ON "session" (("authority_data"->'login'));


COMMIT;
